package im.zego.metaworlddemo.helper.licensehelper;

import android.app.Application;
import android.net.Uri;
import android.util.Log;

import org.jetbrains.annotations.NotNull;

import im.zego.metaworld.ZegoMetaWorldService;

public class ZegoLicenseHelper {

    private static final String TAG = "LicenseAPI";

    private static License mCacheLicense;

    public static void getLicense(Application application, String backendApiUrl, long appID, String appSign, @NotNull final IGetLicenseCallback callback) {

        String authInfo = getAuthInfo(application, appID, appSign);
        backendApiUrl = backendApiUrl + "?Action=DescribeMetaWorldLicense";
        String url = getURL(backendApiUrl, appID, authInfo);
        Log.i(TAG, "url:" + url);
        HttpRequest.asyncGet(url, License.class, (code, message, responseJsonBean) -> {
            if (callback != null) {
                mCacheLicense = responseJsonBean;
                callback.onGetLicense(code, message, responseJsonBean);
            }

        });
    }

    public static License getCacheLicense(){
        return mCacheLicense;
    }

    private static String getAuthInfo(Application application, long appID, String appSign) {
        if(appSign == null){
            throw new RuntimeException("APP_SIGN cannot be null");
        }

        if(appSign.length() != 64){
            throw new RuntimeException("APP_SIGN length must be 64");
        }

        String auth = ZegoMetaWorldService.getInstance().getAuthInfo(application, appID, appSign);
        return auth;
    }

    private static String getURL(String backendApiUrl, long appID,String authInfo){
        Uri.Builder builder = Uri.parse(backendApiUrl).buildUpon();
        builder.appendQueryParameter("AppId", String.valueOf(appID));
        builder.appendQueryParameter("AuthInfo", authInfo);

        return builder.build().toString();
    }

}
