package im.zego.metaworlddemo.view;

import android.content.Context;
import android.content.SharedPreferences;
import android.os.Bundle;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.ProgressBar;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.databinding.DataBindingUtil;

import org.json.JSONException;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.List;

import im.zego.metaworld.ZMWGameObject;
import im.zego.metaworld.ZegoMetaWorldErrorCode;
import im.zego.metaworld.ZegoMetaWorldService;
import im.zego.metaworlddemo.R;
import im.zego.metaworlddemo.databinding.ActivityCabinBinding;
import im.zego.metaworlddemo.helper.licensehelper.HttpRequest;
import im.zego.metaworlddemo.model.Constant;
import im.zego.metaworlddemo.model.DecorData;
import im.zego.metaworlddemo.utils.MetaHomeManager;
import im.zego.metaworlddemo.utils.OKPackageDownloader;
import im.zego.metaworlddemo.view.adapter.DecorAdapter;
import im.zego.metaworlddemo.view.dialog.ConfirmDialog;
import im.zego.room.bean.ZMWHomeState;
import im.zego.room.listener.ZMWEnterCallback;
import im.zego.room.listener.ZMWSwitchHomeStateCallback;

/**
 * 小窝界面
 */
public class CabinActivity extends AppCompatActivity implements ZegoMetaWorldService.MetaWorldServiceDelegate {

    public static final String TAG = "CabinActivity";
    public static final String EXT_ROOM_ASSERT_ID = "roomAssertId";
    public static final String EXT_ROOM_CONFIG = "roomConfig";
    private ActivityCabinBinding binding;
    private List<DecorData.AssetsBean> localAssets = new ArrayList<>();
    private boolean isEditorState = false;
    private String roomAssertId;
    private String roomConfigGG;
    public String userID;
    private boolean mIsRelease = false;
    private DecorAdapter adapter;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        roomAssertId = getIntent().getStringExtra(EXT_ROOM_ASSERT_ID);
        roomConfigGG = getIntent().getStringExtra(EXT_ROOM_CONFIG);
        setFullScreen();
        binding = DataBindingUtil.setContentView(this, R.layout.activity_cabin);

        initView();
        getServerConfig();
        initScene();
    }

    private void initScene() {
        userID = System.currentTimeMillis() + "";
        //添加ZMW初始化回调
        ZegoMetaWorldService.getInstance().addServiceObserver(this);
    }

    private void setFullScreen() {
        this.requestWindowFeature(1);
        this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        this.getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION |
                View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
    }

    private void initView() {
        selectDecorView(binding.tvFurniture);
        if (isGuangGuang()) {
            binding.clDiy.setVisibility(View.GONE);
        }
        binding.ivHomeDecor.setVisibility(View.GONE);
        binding.ivBack.setOnClickListener(view -> {
            if (isEditorState) {
                ConfirmDialog confirmDialog = new ConfirmDialog.Builder(CabinActivity.this)
                        .setContentText("当前装扮尚未保存，是否继续返回首页？")
                        .setCancelText("返回首页", (dialog, i) -> {
                            backInternal();
                        })
                        .setOkText("保存装扮", (dialog, i) -> {
                            dialog.dismiss();
                            ZegoMetaWorldService.getInstance().postMetaWorldJob(() ->
                                    saveConfigToLocal(MetaHomeManager.getInstance().getHomeRoom().getHomeConfig()));
                            switchHomeState(ZMWHomeState.ZMWHomeStateRunState);
                        })
                        .build();
                confirmDialog.show();
            } else {
                backInternal();
            }
        });
        binding.ivHomeDecor.setOnClickListener(view -> {
            if (binding.clDecor.getVisibility() == View.VISIBLE) {
                binding.clDecor.setVisibility(View.GONE);
            } else {
                binding.clDecor.setVisibility(View.VISIBLE);
            }
        });
        View.OnClickListener decorClassClickListener = view -> {
            selectDecorView(view);
            showDecorClass(view);
        };
        binding.tvFurniture.setOnClickListener(decorClassClickListener);
        binding.tvFloor.setOnClickListener(decorClassClickListener);
        binding.tvWall.setOnClickListener(decorClassClickListener);
        binding.tvPlants.setOnClickListener(decorClassClickListener);

        binding.ivGoDiy.setOnClickListener(view -> switchHomeState(ZMWHomeState.ZMWHomeStateEditorState));

        binding.tvEditorSave.setOnClickListener(view -> {
            ZegoMetaWorldService.getInstance().postMetaWorldJob(() ->
                    saveConfigToLocal(MetaHomeManager.getInstance().getHomeRoom().getHomeConfig()));
            switchHomeState(ZMWHomeState.ZMWHomeStateRunState);
        });
    }

    private void backInternal() {
        unInitRoom();
        finish();
    }

    private boolean isGuangGuang() {
        return roomAssertId.startsWith("gg");
    }

    private void selectDecorView(View view) {
        binding.tvFurniture.setSelected(false);
        binding.tvFloor.setSelected(false);
        binding.tvWall.setSelected(false);
        binding.tvPlants.setSelected(false);
        view.setSelected(true);
    }

    private void showDecorClass(View view) {
        int type = 1;
        if (view == binding.tvFurniture) {

        } else if (view == binding.tvFloor) {
            type = 2;
        } else if (view == binding.tvWall) {
            type = 3;
        } else if (view == binding.tvPlants) {
            type = 4;
        }
        adapter = new DecorAdapter(getAssetsByType(type));
        adapter.setOnDecorClickListener((itemView, position, bean) -> {
            if (isEditorState) {
                ProgressBar pbLoading = itemView.findViewById(R.id.progress);
                pbLoading.setVisibility(View.VISIBLE);
                ZegoMetaWorldService.getInstance().postMetaWorldJob(() ->
                        MetaHomeManager.getInstance().getHomeRoom().applyAssetWithAssetId(bean.getId(), errorCode -> {
                            runOnUiThread(() -> {
                                pbLoading.setVisibility(View.GONE);
                                if (errorCode != 0) {
                                    if (errorCode == 9000100) {
                                        Toast.makeText(CabinActivity.this, "已经到达最大数量啦～", Toast.LENGTH_SHORT).show();
                                    } else {
                                        Toast.makeText(CabinActivity.this, "添加物品异常", Toast.LENGTH_SHORT).show();
                                    }
                                }
                            });

                        }));
            }
        });
        binding.rvDecor.setAdapter(adapter);
    }

    private void getServerConfig() {
        HttpRequest.asyncGetOssConfig(Constant.CONFIG_ADDRESS, DecorData.class, (responseJsonBean) -> {
            if (responseJsonBean != null) {
                localAssets = responseJsonBean.getAssets();
                showDecorClass(binding.tvFurniture);
            } else {
                Toast.makeText(this, "服务器异常，请退出重试", Toast.LENGTH_SHORT).show();
            }
        });
    }

    /**
     * type: 1：家具 2：地板 3：墙壁 4：绿植
     */
    private List<DecorData.AssetsBean> getAssetsByType(int type) {
        List<DecorData.AssetsBean> list = new ArrayList<>();
        for (DecorData.AssetsBean bean : localAssets) {
            if (bean.getType() == type) {
                list.add(bean);
            }
        }
        return list;
    }

    /**
     * 进入运行态场景
     */
    private void enterHomeRoom() {
        ZegoMetaWorldService.getInstance().postMetaWorldJob(() -> {
            showLoadingView();
            showEditorButton();
            ZMWEnterCallback zmwEnterCallback = new ZMWEnterCallback() {
                @Override
                public void onResult(int errorCode, ZMWGameObject obj) {
                    Log.d(TAG, "enter run state " + errorCode);
                    runOnUiThread(() -> binding.tvLoadingPb.setText(""));
                    if (errorCode == 0) {
                        showContentView();
                        isEditorState = false;
                        MetaHomeManager.getInstance().attachMetaWorldView(binding.zlwvSceneView);
                    } else {
                        Toast.makeText(CabinActivity.this, "进入小窝场景失败，错误码 " + errorCode, Toast.LENGTH_SHORT).show();
                        finish();
                    }
                }

                @Override
                public void onProgress(float progress) {
                    runOnUiThread(() -> {
                        final int fixProgress = (int) (progress * 100);
                        binding.pbSceneLoading.setProgress(fixProgress);
                        binding.tvLoadingPb.setText("正在加载，耐心等待…   " + fixProgress + "%");
                    });
                }
            };

            String avatarConfigJson = "";
            JSONObject jsonObject = new JSONObject();
            try {
                jsonObject.put("modelID", "GirlXWPreset");
            } catch (JSONException e) {
                e.printStackTrace();
            }
            avatarConfigJson = jsonObject.toString();
            if (isGuangGuang()) {
                MetaHomeManager.getInstance().getHomeRoom().enterWithHomeConfig(
                        roomConfigGG, avatarConfigJson, ZMWHomeState.ZMWHomeStateRunState, zmwEnterCallback);
            } else {
                if (TextUtils.isEmpty(getLocalAssertConfig())) {
                    MetaHomeManager.getInstance().getHomeRoom().enterWithHomeAssetId(
                            roomAssertId, avatarConfigJson, ZMWHomeState.ZMWHomeStateRunState, zmwEnterCallback);
                } else {
                    MetaHomeManager.getInstance().getHomeRoom().enterWithHomeConfig(
                            getLocalAssertConfig(), avatarConfigJson, ZMWHomeState.ZMWHomeStateRunState, zmwEnterCallback);
                }
            }
        });
    }

    /**
     * 切换场景
     */
    private void switchHomeState(ZMWHomeState state) {
        ZegoMetaWorldService.getInstance().postMetaWorldJob(() -> {
            showLoadingView();
            if (state == ZMWHomeState.ZMWHomeStateRunState) {
                showEditorButton();
            } else {
                showSaveButton();
            }
            MetaHomeManager.getInstance().getHomeRoom().switchHomeState(state, new ZMWSwitchHomeStateCallback() {
                @Override
                public void onResult(int errorCode, ZMWGameObject roomObj) {
                    runOnUiThread(() -> binding.tvLoadingPb.setText(""));
                    if (errorCode == 0) {
                        MetaHomeManager.getInstance().attachMetaWorldView(binding.zlwvSceneView);
                        showContentView();
                        if (state == ZMWHomeState.ZMWHomeStateEditorState) {
                            isEditorState = true;
                            runOnUiThread(() -> {
                                binding.ivHomeDecor.setVisibility(View.VISIBLE);
                                binding.clDecor.setVisibility(View.VISIBLE);
                            });
                        } else {
                            isEditorState = false;
                            runOnUiThread(() -> {
                                binding.ivHomeDecor.setVisibility(View.GONE);
                                binding.clDecor.setVisibility(View.GONE);
                            });
                        }
                    }
                }

                @Override
                public void onProgress(float progress) {
                    runOnUiThread(() -> {
                        final int fixProgress = (int) (progress * 100);
                        binding.pbSceneLoading.setProgress(fixProgress);
                        binding.tvLoadingPb.setText("正在加载，耐心等待…   " + fixProgress + "%");
                    });
                }
            });
        });
    }

    private void showContentView() {
        runOnUiThread(() -> {
            binding.clLoading.setVisibility(View.GONE);
            binding.clContent.setVisibility(View.VISIBLE);
        });
    }

    private void showLoadingView() {
        runOnUiThread(() -> {
            binding.clLoading.setVisibility(View.VISIBLE);
            binding.clContent.setVisibility(View.INVISIBLE);
        });
    }

    private void showSaveButton() {
        runOnUiThread(() -> {
            if (!isGuangGuang()) {
                binding.ivGoDiy.setVisibility(View.GONE);
                binding.tvEditorSave.setVisibility(View.VISIBLE);
            }
        });
    }

    private void showEditorButton() {
        runOnUiThread(() -> {
            if (!isGuangGuang()) {
                binding.ivGoDiy.setVisibility(View.VISIBLE);
                binding.tvEditorSave.setVisibility(View.GONE);
            }
        });
    }

    private void saveConfigToLocal(String assertConfig) {
        SharedPreferences preferences = getSharedPreferences("SmallRoom", Context.MODE_PRIVATE);
        SharedPreferences.Editor editor = preferences.edit();
        editor.putString("assert_config", assertConfig);
        editor.apply();
    }

    private String getLocalAssertConfig() {
        SharedPreferences preferences = getSharedPreferences("SmallRoom", Context.MODE_PRIVATE);
        return preferences.getString("assert_config", "");
    }

    @Override
    protected void onDestroy() {
        unInitRoom();
        super.onDestroy();
    }

    @Override
    protected void onStop() {
        super.onStop();
        if (isFinishing()) {
            unInitRoom();
        }
    }

    private void unInitRoom() {
        if (mIsRelease) {
            return;
        }
        ZegoMetaWorldService.getInstance().postMetaWorldJob(() -> {
            binding.zlwvSceneView.detachCamera();
            MetaHomeManager.getInstance().destroyRoom();
            ZegoMetaWorldService.getInstance().removeServiceObserver(this);
        });
        mIsRelease = true;
    }

    @Override
    public void onError(ZegoMetaWorldErrorCode code, String desc) {

    }

    @Override
    public void onStateChange(ZegoMetaWorldService.MetaWorldServiceState state) {
        if (state == ZegoMetaWorldService.MetaWorldServiceState.MetaWorld_InitSucceed) {
            ZegoMetaWorldService.getInstance().postMetaWorldJob(() -> {
                OKPackageDownloader downloader = OKPackageDownloader.getInstance();
                downloader.active();
                enterHomeRoom();
            });
        }
    }
}