package im.zego.metaworlddemo.view;

import android.content.Intent;
import android.content.IntentFilter;
import android.net.ConnectivityManager;
import android.net.wifi.WifiManager;
import android.os.Bundle;
import android.os.Handler;
import android.os.Looper;
import android.text.TextUtils;
import android.util.Log;
import android.view.View;
import android.view.Window;
import android.view.WindowManager;
import android.widget.Toast;

import androidx.appcompat.app.AppCompatActivity;
import androidx.constraintlayout.widget.ConstraintLayout;
import androidx.databinding.DataBindingUtil;

import im.zego.metaworld.ZegoMetaWorldService;
import im.zego.metaworld.ZegoMetaWorldServiceConfig;
import im.zego.metaworlddemo.R;
import im.zego.metaworlddemo.databinding.ActivityCabinMainBinding;
import im.zego.metaworlddemo.helper.licensehelper.HttpRequest;
import im.zego.metaworlddemo.helper.licensehelper.ZegoLicenseHelper;
import im.zego.metaworlddemo.model.Constant;
import im.zego.metaworlddemo.utils.DimenUtils;
import im.zego.metaworlddemo.utils.NetworkReceiver;
import im.zego.metaworlddemo.view.dialog.LoadingDialog;
import im.zego.room.bean.HomeConstant;

/**
 * 小窝 - 界面
 */
public class CabinMainActivity extends AppCompatActivity {

    public static final String TAG = "CabinActivity";
    private ActivityCabinMainBinding binding;
    private boolean isTemplate = false;
    private String roomConfigGG01;

    public static final String licenseBackendURL = "http://zego-meta-api.zego.im";

    private boolean authSuccess = false;
    private NetworkReceiver netWorkReceiver;
    private LoadingDialog loadingDialog;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setFullScreen();
        binding = DataBindingUtil.setContentView(this, R.layout.activity_cabin_main);

        initMetaConfig();

        // 注册网络状态监听广播
        netWorkReceiver = new NetworkReceiver();
        IntentFilter filter = new IntentFilter();
        filter.addAction(WifiManager.WIFI_STATE_CHANGED_ACTION);
        filter.addAction(WifiManager.NETWORK_STATE_CHANGED_ACTION);
        filter.addAction(ConnectivityManager.CONNECTIVITY_ACTION);
        registerReceiver(netWorkReceiver, filter);
        netWorkReceiver.setnetworkListener(this::initMetaConfig);

        binding.tvDiy.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                isTemplate = false;
                binding.ivPreview.setImageResource(R.mipmap.xw_div_home_preview);
                binding.ivPreview2.setImageResource(R.mipmap.xw_developing);

                binding.tvDiy.setBackgroundResource(R.mipmap.xw_room_title_bg_selected);
                binding.tvDiy.setTranslationZ(2f);
                ConstraintLayout.LayoutParams layoutParamsDiy = (ConstraintLayout.LayoutParams) binding.tvDiy.getLayoutParams();
                layoutParamsDiy.width = DimenUtils.dp2px(CabinMainActivity.this, 224);
                layoutParamsDiy.height = DimenUtils.dp2px(CabinMainActivity.this, 50);
                binding.tvDiy.setLayoutParams(layoutParamsDiy);

                binding.tvTemplate.setBackgroundResource(R.mipmap.xw_room_title_right_bg);
                binding.tvTemplate.setTranslationZ(1f);
                ConstraintLayout.LayoutParams layoutParamsTemplate = (ConstraintLayout.LayoutParams) binding.tvTemplate.getLayoutParams();
                layoutParamsTemplate.width = DimenUtils.dp2px(CabinMainActivity.this, 197);
                layoutParamsTemplate.height = DimenUtils.dp2px(CabinMainActivity.this, 44);
                binding.tvTemplate.setLayoutParams(layoutParamsTemplate);
            }
        });
        binding.tvTemplate.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                isTemplate = true;
                binding.ivPreview.setImageResource(R.mipmap.xw_div_home_template_preview);
                binding.ivPreview2.setImageResource(R.mipmap.xw_developing);

                binding.tvDiy.setBackgroundResource(R.mipmap.xw_room_title_left_bg);
                binding.tvDiy.setTranslationZ(1f);
                ConstraintLayout.LayoutParams layoutParamsDiy = (ConstraintLayout.LayoutParams) binding.tvDiy.getLayoutParams();
                layoutParamsDiy.width = DimenUtils.dp2px(CabinMainActivity.this, 197);
                layoutParamsDiy.height = DimenUtils.dp2px(CabinMainActivity.this, 44);
                binding.tvDiy.setLayoutParams(layoutParamsDiy);

                binding.tvTemplate.setBackgroundResource(R.mipmap.xw_room_title_bg_selected);
                binding.tvTemplate.setTranslationZ(2f);
                ConstraintLayout.LayoutParams layoutParamsTemplate = (ConstraintLayout.LayoutParams) binding.tvTemplate.getLayoutParams();
                layoutParamsTemplate.width = DimenUtils.dp2px(CabinMainActivity.this, 224);
                layoutParamsTemplate.height = DimenUtils.dp2px(CabinMainActivity.this, 50);
                binding.tvTemplate.setLayoutParams(layoutParamsTemplate);
            }
        });

        binding.ivPreview.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View view) {
                if (!authSuccess) {
                    LoadingDialog loadingDialog = new LoadingDialog(CabinMainActivity.this);
                    loadingDialog.show();
                    ZegoLicenseHelper.getLicense(MWApplication.application,
                            licenseBackendURL, Constant.appID, Constant.appSign,
                            (code, message, license) -> {
                                loadingDialog.dismiss();
                                Log.i(TAG, "Auth code:" + code + " message:" + message);
                                if (code == 0) {
                                    authSuccess = true;
                                    ZegoMetaWorldServiceConfig serviceConfig = new ZegoMetaWorldServiceConfig(license.getLicense());
                                    ZegoMetaWorldService.getInstance().initWithConfig(MWApplication.application, serviceConfig);
                                    startCabinActivity();
                                } else {
                                    Toast.makeText(CabinMainActivity.this, "鉴权失败，请重试", Toast.LENGTH_SHORT).show();
                                }
                            });
                } else {
                    startCabinActivity();
                }
            }
        });

        getGuangGuangConfig();
    }

    private void setFullScreen() {
        this.requestWindowFeature(1);
        this.requestWindowFeature(Window.FEATURE_NO_TITLE);
        this.getWindow().setFlags(WindowManager.LayoutParams.FLAG_FULLSCREEN, WindowManager.LayoutParams.FLAG_FULLSCREEN);
        this.getWindow().getDecorView().setSystemUiVisibility(View.SYSTEM_UI_FLAG_HIDE_NAVIGATION | View.SYSTEM_UI_FLAG_IMMERSIVE_STICKY);
    }

    private void startCabinActivity() {
        Intent intent = new Intent(CabinMainActivity.this, CabinActivity.class);
        if (isTemplate) {
            if (TextUtils.isEmpty(roomConfigGG01)) {
                Toast.makeText(CabinMainActivity.this, "获取资源失败，请退出重试", Toast.LENGTH_SHORT).show();
                return;
            }
            intent.putExtra(CabinActivity.EXT_ROOM_ASSERT_ID, HomeConstant.HOME_ASSERT_ID_GG01);
            intent.putExtra(CabinActivity.EXT_ROOM_CONFIG, roomConfigGG01);
        } else {
            intent.putExtra(CabinActivity.EXT_ROOM_ASSERT_ID, HomeConstant.HOME_ASSERT_ID_ZB01);
        }
        startActivity(intent);
    }

    private void getGuangGuangConfig() {
        HttpRequest.asyncGetGuangGuangConfig(Constant.GG01, (responseJsonBean) -> {
            roomConfigGG01 = responseJsonBean;
        });
    }

    @Override
    protected void onDestroy() {
        super.onDestroy();
        if (netWorkReceiver != null) {
            unregisterReceiver(netWorkReceiver);
        }
    }

    private void initMetaConfig() {
        if (authSuccess || (loadingDialog != null && loadingDialog.isShowing())) {
            return;
        }
        // 每个world一个url
        loadingDialog = new LoadingDialog(CabinMainActivity.this);
        loadingDialog.show();

        Handler handler = new Handler(Looper.getMainLooper());
        handler.postDelayed(new Runnable() {
            @Override
            public void run() {
                if (loadingDialog.isShowing()) {
                    loadingDialog.dismiss();
                    Toast.makeText(CabinMainActivity.this, "鉴权失败", Toast.LENGTH_SHORT).show();
                }
            }
        }, 10 * 1000);
        ZegoLicenseHelper.getLicense(MWApplication.application, licenseBackendURL, Constant.appID, Constant.appSign,
                (code, message, license) -> {
                    Log.i(TAG, "Auth code:" + code + " message:" + message);
                    if (code == 0) {
                        loadingDialog.dismiss();
                        authSuccess = true;
                        ZegoMetaWorldServiceConfig serviceConfig = new ZegoMetaWorldServiceConfig(license.getLicense());
                        ZegoMetaWorldService.getInstance().initWithConfig(MWApplication.application, serviceConfig);
                    }
                });
    }
}