package im.zego.metaworlddemo.view.dialog;

import android.content.Context;
import android.content.DialogInterface;
import android.os.Bundle;
import android.text.TextUtils;
import android.view.View;
import android.view.WindowManager;

import im.zego.metaworlddemo.databinding.DialogConfirmBinding;

/**
 * 两个按钮的弹窗样式
 */
public class ConfirmDialog extends BaseDialog<DialogConfirmBinding> {

    private String titleText;
    private String contentText;
    private String okText;
    private String cancelText;
    private DialogInterface.OnClickListener positiveListener;
    private DialogInterface.OnClickListener negativeListener;

    public ConfirmDialog(Builder builder) {
        super(builder.context);
        this.titleText = builder.titleText;
        this.contentText = builder.contentText;
        this.okText = builder.okText;
        this.cancelText = builder.cancelText;
        this.positiveListener = builder.positiveListener;
        this.negativeListener = builder.negativeListener;
        getWindow().addFlags(WindowManager.LayoutParams.FLAG_DIM_BEHIND);
    }

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        if (!TextUtils.isEmpty(titleText)) {
            binding.confirmTitle.setText(titleText);
        }
        if (!TextUtils.isEmpty(contentText)) {
            binding.confirmContent.setText(contentText);
        }
        if (!TextUtils.isEmpty(okText)) {
            binding.confirmOk.setText(okText);
        }
        if (!TextUtils.isEmpty(cancelText)) {
            binding.confirmCancel.setText(cancelText);
        }
        binding.confirmCancel.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (negativeListener != null) {
                    negativeListener.onClick(ConfirmDialog.this, DialogInterface.BUTTON_NEGATIVE);
                }
            }
        });
        binding.confirmOk.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (positiveListener != null) {
                    positiveListener.onClick(ConfirmDialog.this, DialogInterface.BUTTON_POSITIVE);
                }
            }
        });
    }

    public static class Builder {

        private String titleText;
        private String contentText;
        private String okText;
        private String cancelText;
        private Context context;
        private DialogInterface.OnClickListener positiveListener;
        private DialogInterface.OnClickListener negativeListener;

        public Builder(Context context) {
            this.context = context;
        }

        public Builder setTitleText(String titleText) {
            this.titleText = titleText;
            return this;
        }

        public Builder setContentText(String contentText) {
            this.contentText = contentText;
            return this;
        }

        public Builder setOkText(String okText, DialogInterface.OnClickListener listener) {
            this.okText = okText;
            this.positiveListener = listener;
            return this;
        }

        public Builder setCancelText(String cancelText, DialogInterface.OnClickListener listener) {
            this.cancelText = cancelText;
            this.negativeListener = listener;
            return this;
        }

        public ConfirmDialog build() {
            ConfirmDialog dialog = new ConfirmDialog(this);
            return dialog;
        }
    }
}
